#include <string.h>
#ifdef DREAMCAST
#include <kos.h>
#include <GL/gl.h>
#else
#include <windows.h>
#include <GL\gl.h>
#endif
#include <malloc.h>
#include "window.h"
#include "font.h"
#include "input.h"

int next_list_id=0;

int c99_create_list(int win_id, int x, int y, int xsize, int ysize) {
  struct C99Window *current_win=get_window(win_id);
  struct C99ListBox *current=current_win->listbox_list;
  struct C99ListBox *prev=NULL;
  while(current) {
    prev=current;
    current=current->next;
  }
  current=(struct C99ListBox*)malloc(sizeof(C99ListBox));
  current->x=x;
  current->y=y;
  current->sel=-1;
  current->onChange=NULL;
  current->itemcnt=0;
  current->list_id=next_list_id;
  current->win_id=win_id;
  current->xsize=xsize;
  current->ysize=ysize;
  current->next=NULL;
  current->onClick=NULL;
  if(prev) {
    prev->next=current;
  } else {
    current_win->listbox_list=current;
  }
  return next_list_id++;
}

struct C99ListBox *get_list(int win_id, int list_id) {
  struct C99Window *cur_win=get_window(win_id);
  struct C99ListBox *current=cur_win->listbox_list;

  while(current) {
    if(current->list_id==list_id) return current;
    current=current->next;
  }

  return NULL;
}

void c99_update_list(struct C99ListBox *current) {
  int oldsel=current->sel;
  switch(poll_game_device(0)) {
  case MOVE_UP:
    current->sel--;
    if(current->sel<0) current->sel=0;
    break;
  case MOVE_DOWN:
    current->sel++;
    if(current->sel>=current->itemcnt) current->sel=current->itemcnt-1;
    break;
  }
  if(current->onChange!=NULL && oldsel!=current->sel) {
    current->onChange(current->win_id,current->list_id);
  }
}

void c99_draw_list(struct C99ListBox *current) {
  int i,off=0,max;

  glDisable(GL_TEXTURE_2D);
  glColor3f(0,0,0);
  glBegin(GL_QUADS);
  glVertex3f(current->x,current->y,0);
  glVertex3f(current->x+current->xsize,current->y,0);
  glVertex3f(current->x+current->xsize,current->y+current->ysize,0);
  glVertex3f(current->x,current->y+current->ysize,0);
  glEnd();
  glColor3f(1,1,1);
  glBegin(GL_QUADS);
  glVertex3f(current->x+2,current->y+2,0);
  glVertex3f(current->x+current->xsize-2,current->y+2,0);
  glVertex3f(current->x+current->xsize-2,current->y+current->ysize-2,0);
  glVertex3f(current->x+2,current->y+current->ysize-2,0);
  glEnd();
  if(current->sel>=0) {
    if(current->sel>=(current->ysize/16)) {
      off=current->sel-(current->ysize/16)+1;
    } else {
      off=0;
    }
    glColor3f(0,0,0);
    glBegin(GL_QUADS);
    glVertex3f(current->x+2,current->y+2+(current->sel-off)*16,0.1);
    glVertex3f(current->x+current->xsize-2,current->y+2+(current->sel-off)*16,0.1);
    glVertex3f(current->x+current->xsize-2,current->y+18+(current->sel-off)*16,0.1);
    glVertex3f(current->x+2,current->y+18+(current->sel-off)*16,0.1);
    glEnd();
  }

  //fugly version: for(i=0;i<((current->itemcnt>=current->ysize/16)?(current->ysize/16):(current->itemcnt));i++) {
  for(i=0;i<current->itemcnt && i<current->ysize/16;i++) {
    if(current->sel>=0 && i+off==current->sel) {
      glColor3f(1,1,1);
    } else {
      glColor3f(0,0,0);
    }
    glEnable(GL_TEXTURE_2D);
    draw_string(current->items[i+off],current->x+2,current->y+i*16+2,0.2);
  }
}

void c99_list_add_item(int win_id, int list_id, char *text) {
  struct C99ListBox *current=get_list(win_id,list_id);
  strcpy(current->items[current->itemcnt++],text);
}

void c99_list_set_changed(int win_id, int list_id, list_change_callback cb) {
  struct C99ListBox *current=get_list(win_id,list_id);
  current->onChange=cb;
}

void c99_list_set_click(int win_id, int list_id, list_click_callback cb) {
  struct C99ListBox *current=get_list(win_id,list_id);
  current->onClick=cb;
}

void c99_list_empty(int win_id, int list_id) {
  struct C99ListBox *current=get_list(win_id,list_id);

  current->itemcnt=0;
  current->sel=-1;
}