#include <string.h>
#ifdef WIN32
#include <windows.h>
#include <stdio.h>
#include <GL\gl.h>
#else
#include <kos.h>
#include <GL/gl.h>
#endif
#include <malloc.h>
#include "window.h"
#include "font.h"
#include "input.h"

struct C99Window *window_list=NULL;

int next_win_id=0;

void c99_window_click(int id, int x, int y) {
  struct C99Window *current=get_window(id);

  free(current->title);
  current->title=(char *)malloc(strlen("Clicky")+1);
  strcpy(current->title,"Clicky");
}

int c99_create_window(int x, int y, int xsize, int ysize, char *title) {
  struct C99Window *current=window_list;
  struct C99Window *prev=NULL;
  while(current) {
    prev=current;
    current=current->next;
  }
  current=(struct C99Window*)malloc(sizeof(struct C99Window));
  current->x=x;
  current->y=y;
  current->id=next_win_id;
  current->xsize=xsize;
  current->ysize=ysize;
  current->title=(char *)malloc(strlen(title)+1);
  current->next=NULL;
  current->button_list=NULL;
  current->listbox_list=NULL;
  current->inputbox_list=NULL;
  current->label_list=NULL;
  current->onClick=c99_window_click;
  strcpy(current->title,title);
  if(prev) {
    prev->next=current;
  } else {
    window_list=current;
  }
  return next_win_id++;
}

struct C99Window *get_window(int id) {
  struct C99Window *current=window_list;

  while(current) {
    if(current->id==id) return current;
    current=current->next;
  }

  return NULL;
}

void c99_draw_window(struct C99Window *current) {
  struct C99Button *current_but=NULL;
  struct C99ListBox *current_list=NULL;
  struct C99InputBox *current_inputbox=NULL;
  struct C99Label *current_label=NULL;

  //printf("Drawing a window..\n");
  if(current==NULL) return;
  //printf("x: %i y: %i title: %s\n",current->x, current->y, current->title);
  //glLoadIdentity();
  glTranslatef(current->x,current->y+16,0.1);
  glDisable(GL_TEXTURE_2D);
  glColor4f(0,0,0,0.5f);
  glEnable(GL_BLEND);
  glBegin(GL_QUADS);
  glVertex3f(8,-16+8,0);
  glVertex3f(current->xsize+8,-16+8,0);
  glVertex3f(current->xsize+8,current->ysize+8,0);
  glVertex3f(8,current->ysize+8,0);
  glEnd();
  glColor3f(0,0,0.6f);
  glBegin(GL_QUADS);
  glVertex3f(0,-16,0);
  glVertex3f(current->xsize,-16,0);
  glVertex3f(current->xsize,current->y,0);
  glVertex3f(0,current->y,0);
  glEnd();
  glColor3f(1,1,1);
  glBegin(GL_QUADS);
  glVertex3f(0,0,0);
  glVertex3f(0+current->xsize,0,0);
  glVertex3f(0+current->xsize,0+current->ysize,0);
  glVertex3f(0,0+current->ysize,0);
  glEnd();
  glEnable(GL_TEXTURE_2D);
  draw_string(current->title,(current->xsize/2.0f)-(strlen(current->title)*get_font_width())/2.0f,-16,0.1);
  current_but=current->button_list;
  while(current_but) {
    c99_draw_button(current_but);
    current_but=current_but->next;
  }
  current_list=current->listbox_list;
  while(current_list) {
    c99_draw_list(current_list);
    current_list=current_list->next;
  }
  current_inputbox=current->inputbox_list;
  while(current_inputbox) {
    c99_update_inputbox(current_inputbox);
    c99_draw_inputbox(current_inputbox);
    current_inputbox=current_inputbox->next;
  }
  current_label=current->label_list;
  while(current_label) {
    c99_draw_label(current_label);
    current_label=current_label->next;
  }
#ifdef WIN32
  glDisable(GL_BLEND);
#endif
  glLoadIdentity();
}

void set_window_click(int id, window_click_callback onClick) {
  struct C99Window *current=get_window(id);
  current->onClick=onClick;
}

int mousex=320,mousey=240,lmb=0;

void render_windows() {
  struct C99Window *current_win=window_list;
  struct C99Button *current_button=NULL;
  struct C99ListBox *current_list=NULL;

  set_echo(0);
  set_show_cursor(0);
  read_mouse(&mousex,&mousey,&lmb);
  glDisable(GL_TEXTURE_2D);
  glColor4f(0,0,0,0.8);
#ifdef WIN32
  glEnable(GL_BLEND);
#endif
  glBegin(GL_TRIANGLES);
  glVertex3f(mousex-2,mousey-4,0.3);
  glVertex3f(mousex+18,mousey+16,0.3);
  glVertex3f(mousex-2,mousey+24,0.3);
  glEnd();
#ifdef WIN32
  glDisable(GL_BLEND);
#endif
  glColor3f(0,0.6,0.8);
  glBegin(GL_TRIANGLES);
  glVertex3f(mousex,mousey,0.31);
  glVertex3f(mousex+15,mousey+15,0.31);
  glVertex3f(mousex,mousey+21,0.31);
  glEnd();
  glEnable(GL_TEXTURE_2D);

  while(current_win) {
    current_button=current_win->button_list;
    while(current_button) {
      if(lmb&&mousex-current_win->x>current_button->x &&
        mousex-current_win->x<(current_button->x+current_button->xsize) &&
        mousey-current_win->y>current_button->y+16 &&
        mousey-current_win->y<(current_button->y+current_button->ysize+16)) {
          current_button->highlight=1;
      } else {
        if(current_button->highlight) {
          if(!lmb&&current_button->onClick) {
            current_button->onClick(current_win->id,current_button->but_id);
          }
          current_button->highlight=0;
        }
      }
      current_button=current_button->next;
    }
    current_list=current_win->listbox_list;
    while(current_list) {
      c99_update_list(current_list);
      if(lmb&&mousex-current_win->x>current_list->x &&
        mousex-current_win->x<(current_list->x+current_list->xsize) &&
        mousey-current_win->y>current_list->y+16 &&
        mousey-current_win->y<(current_list->y+current_list->ysize+16)) {
        current_list->sel=(mousey-current_win->y-current_list->y-16)/16;
        if(current_list->sel>=current_list->itemcnt) current_list->sel=-1;
        if(current_list->onChange) current_list->onChange(current_list->win_id,current_list->list_id);
        if(current_list->onClick) current_list->onClick(current_list->win_id,current_list->list_id);
      }
      current_list=current_list->next;
    }
    c99_draw_window(current_win);
    current_win=current_win->next;
  }
  //set_echo(1);
  set_show_cursor(1);
}

void delete_window(int w) {
  struct C99Window *current=get_window(w); 
  struct C99Button *current_but=NULL;
  struct C99Button *prev_but=NULL;
  struct C99ListBox *current_list=NULL;
  struct C99ListBox *prev_list=NULL;
  struct C99InputBox *current_inputbox=NULL;
  struct C99InputBox *prev_inputbox=NULL;

  current_but=current->button_list;
  while(current_but) {
    prev_but=current_but;
    current_but=current_but->next;
    free(prev_but->label);
    free(prev_but);
  }

  current_list=current->listbox_list;
  while(current_list) {
    prev_list=current_list;
    current_list=current_list->next;
    free(prev_list);
  }

  current_inputbox=current->inputbox_list;
  while(current_inputbox) {
    prev_inputbox=current_inputbox;
    current_inputbox=current_inputbox->next;
    free(prev_inputbox);
  }

  free(current->title);
  free(current);

  window_list=NULL;
}